#!/usr/bin/env python

#
# Seccomp Library test program
#
# Copyright (c) 2019 Oracle and/or its affiliates.  All rights reserved.
# Author: Tom Hromatka <tom.hromatka@oracle.com>
#

#
# This library is free software; you can redistribute it and/or modify it
# under the terms of version 2.1 of the GNU Lesser General Public License as
# published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses>.
#

import argparse
import sys

import util

from seccomp import *

def test(args):
    set_api(1)
    f = SyscallFilter(ERRNO(100))
    f.remove_arch(Arch())
    f.add_arch(Arch("x86_64"))

    # libseccomp utilizes a hash table to manage BPF blocks.  It currently
    # employs MurmurHash3 where the key is the hashed values of the BPF
    # instruction blocks, the accumulator start, and the accumulator end.
    # Changes to the hash algorithm will likely affect this test.

    # The following rules were derived from an issue reported by Tor:
    # https://github.com/seccomp/libseccomp/issues/148
    #
    # In the steps below, syscall 1001 is configured similarly to how
    # Tor configured socket.  The fairly complex rules below led to
    # a hash collision with rt_sigaction (syscall 1000) in this test.

    f.add_rule_exactly(ALLOW, 1001, Arg(0, EQ, 1), Arg(1, MASKED_EQ, 0xf, 2),
                       Arg(2, EQ, 3))
    f.add_rule_exactly(ALLOW, 1001, Arg(0, EQ, 1), Arg(1, MASKED_EQ, 0xf, 1))
    f.add_rule_exactly(ALLOW, 1000, Arg(0, EQ, 2))
    f.add_rule_exactly(ALLOW, 1000, Arg(0, EQ, 1))
    return f

args = util.get_opt()
ctx = test(args)
util.filter_output(args, ctx)

# kate: syntax python;
# kate: indent-mode python; space-indent on; indent-width 4; mixedindent off;



#!/usr/bin/env python

#
# Seccomp Library test program
#
# Copyright (c) 2018 Oracle and/or its affiliates.  All rights reserved.
# Author: Tom Hromatka <tom.hromatka@oracle.com>
#

#
# This library is free software; you can redistribute it and/or modify it
# under the terms of version 2.1 of the GNU Lesser General Public License as
# published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses>.
#

import argparse
import sys

import util

from seccomp import *

denylist = [
    "times",
    "ptrace",
    "getuid",
    "syslog",
    "getgid",
    "setuid",
    "setgid",
    "geteuid",
    "getegid",
    "setpgid",
    "getppid",
    "getpgrp",
    "setsid",
    "setreuid",
    "setregid",
    "getgroups",
    "setgroups",
    "setresuid",
    "getresuid",
    "setresgid",
    "getresgid",
    "getpgid",
    "setfsuid",
    "setfsgid",
]

def test():
    action = util.parse_action(sys.argv[1])
    if not action == ALLOW:
        quit(1)
    util.install_trap()
    f = SyscallFilter(TRAP)
    f.set_attr(Attr.CTL_TSYNC, 1)
    f.set_attr(Attr.CTL_OPTIMIZE, 2)
    # NOTE: additional syscalls required for python
    f.add_rule(ALLOW, "stat")
    f.add_rule(ALLOW, "fstat")
    f.add_rule(ALLOW, "open")
    f.add_rule(ALLOW, "openat")
    f.add_rule(ALLOW, "mmap")
    f.add_rule(ALLOW, "munmap")
    f.add_rule(ALLOW, "read")
    f.add_rule(ALLOW, "write")
    f.add_rule(ALLOW, "close")
    f.add_rule(ALLOW, "rt_sigaction")
    f.add_rule(ALLOW, "rt_sigreturn")
    f.add_rule(ALLOW, "sigreturn")
    f.add_rule(ALLOW, "sigaltstack")
    f.add_rule(ALLOW, "brk")
    f.add_rule(ALLOW, "exit_group")

    for syscall in denylist:
        f.add_rule(KILL, syscall)

    f.load()
    try:
        util.write_file("/dev/null")
    except OSError as ex:
        quit(ex.errno)
    quit(160)

test()

# kate: syntax python;
# kate: indent-mode python; space-indent on; indent-width 4; mixedindent off;



#!/usr/bin/env python

#
# Seccomp Library test program
#
# Copyright (c) 2012 Red Hat <pmoore@redhat.com>
# Author: Paul Moore <paul@paul-moore.com>
#

#
# This library is free software; you can redistribute it and/or modify it
# under the terms of version 2.1 of the GNU Lesser General Public License as
# published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses>.
#

import argparse
import sys

import util

from seccomp import *

def test():
    # this test differs from the native test for obvious reasons
    try:
        f = SyscallFilter(ALLOW + 1)
    except RuntimeError:
        pass

    f = SyscallFilter(ALLOW)
    try:
        f.reset(KILL + 1)
    except ValueError:
        pass

    f = SyscallFilter(ALLOW)
    try:
        f.syscall_priority(-10000, 1)
    except RuntimeError:
        pass

    f = SyscallFilter(ALLOW)
    try:
        f.add_rule(ALLOW, "read")
    except RuntimeError:
        pass
    try:
        f.add_rule(KILL - 1, "read")
    except RuntimeError:
        pass
    try:
        f.add_rule(KILL, "read",
                Arg(0, EQ, 0),
                Arg(1, EQ, 1),
                Arg(2, EQ, 2),
                Arg(3, EQ, 3),
                Arg(4, EQ, 4),
                Arg(5, EQ, 5),
                Arg(6, EQ, 6),
                Arg(7, EQ, 7))
    except RuntimeError:
        pass
    try:
        f.add_rule(KILL, -1001)
    except RuntimeError:
        pass

    f = SyscallFilter(ALLOW)
    f.remove_arch(Arch())
    f.add_arch(Arch("x86"))
    try:
        f.add_rule_exactly(KILL, "socket", Arg(0, EQ, 2))
    except RuntimeError:
        pass

    f = SyscallFilter(ALLOW)
    try:
        f.add_rule(ERRNO(0xffff), "read")
    except RuntimeError:
        pass

test()

# kate: syntax python;
# kate: indent-mode python; space-indent on; indent-width 4; mixedindent off;
